==> appdynamics-0.1.1.tgz: package/lib/proxy/proxy-transport.js <==
'use strict';

var util = require('util');
var os = require('os');
var fs = require('fs');
var path = require('path');
var zmq = require('appdynamics-zmq');
var Schema = require('appdynamics-protobuf').Schema;
var ProtobufModel = require('./protobuf-model').ProtobufModel;

function ProxyTransport(agent) {
  this.agent = agent;

  this.schema = undefined;
  this.configSock = undefined;
  this.reportingSocket = undefined;
  this.infoSocket = undefined;

  this.protobufModel = undefined;
}
exports.ProxyTransport = ProxyTransport;


ProxyTransport.prototype.init = function() {
  var self = this;

  self.protobufModel = new ProtobufModel(self.agent);
  self.protobufModel.init();

  self.nextMessageId = 0;
  self.transactionMap = {};

  self.agent.on('proxyStarted', function(nodeIndex) {
    try {
      self.start(nodeIndex);
    }
    catch(err) {
      self.agent.logger.error(err);
    }
  });
}


ProxyTransport.prototype.cleanupTransactionMap = function() {
  var self = this;

  var minTs = Date.now() - 10000;
  for(var requestId in self.transactionMap) {
    var transaction = self.transactionMap[requestId];
    if(transaction.ts < minTs) {
      delete self.transactionMap[requestId];
    }
  }
}


ProxyTransport.prototype.start = function(nodeIndex) {
  var self = this;

  // proxy communication directory for zmq
  var opts = self.agent.opts;
  //var proxyCommDir = self.agent.tmpDir + '/proxy/' + nodeIndex;
  var proxyCommDir = self.agent.tmpDir + '/proxy/' + opts.applicationName + '/' + opts.tierName +  '/' + nodeIndex;


  // init transport and response listeners
  self.schema = new Schema(fs.readFileSync(path.join(__dirname, '../..', 'conf/protobuf.desc')));
  self.ASyncRequest = self.schema['appdynamics.pb.ASyncRequest'];
  self.ConfigResponse = self.schema['appdynamics.pb.ConfigResponse'];
  self.BTInfoRequest = self.schema['appdynamics.pb.BTInfoRequest'];
  self.BTInfoResponse = self.schema['appdynamics.pb.BTInfoResponse'];
  self.ASyncMessage = self.schema['appdynamics.pb.ASyncMessage'];

  self.configSocket = zmq.socket('router');
  self.configSocket.connect('ipc://' + proxyCommDir + '/0');
  self.configSocket.on('message', function(envelope, empty, part1) {
    self.agent.logger.log('config response message received from proxy');
    self.receiveConfigResponse(part1);
  });
  self.configSocket.on('error', function(err) {
    self.agent.logger.error(err);
  });

  self.infoSocket = zmq.socket('router');
  self.infoSocket.connect('ipc://' + proxyCommDir + '/0');
  self.infoSocket.on('message', function(envelope, empty, part1) {
    self.agent.logger.log('info response message received from proxy');
    self.receiveBTInfoResponse(part1);
  });
  self.infoSocket.on('error', function(err) {
    self.agent.logger.error(err);
  });

  self.reportingSocket = zmq.socket('pub');
  self.reportingSocket.connect('ipc://' + proxyCommDir + '/1');
  self.reportingSocket.on('error', function(err) {
    self.agent.logger.error(err);
  });

  // send on connection, do not wait
  self.agent.timers.setTimeout(function() {
    self.sendConfigRequest();
  }, 2000);

  // should we also reqest config after registering unregistered?
  self.agent.timers.setInterval(function() {
    self.sendConfigRequest();
  }, 30000);

  // send on connection, do not wait
  self.agent.timers.setInterval(function() {
    self.cleanupTransactionMap();
  }, 10000);
}


ProxyTransport.prototype.sendConfigRequest = function() {
  var self = this;

  var lastConfigVersion = self.agent.configManager.getConfigVersion();
  var processMetrics = self.agent.metricsManager.getProcessMetrics();

  var asyncRequestObj = {
    type: 'CONFIG',
    configReq: {
      lastVersion: lastConfigVersion,
      nodejsProcessMetrics: processMetrics
    }
  };

  try {
    self.agent.logger.log(util.inspect(asyncRequestObj, {depth: 20}));
    var asyncRequestBytes = self.ASyncRequest.serialize(asyncRequestObj);

    self.configSocket.send([
      "AsyncReqRouter",
      "",
      asyncRequestBytes
    ]);
  }
  catch(err) {
    self.agent.logger.error(err);
  }
}


ProxyTransport.prototype.receiveConfigResponse = function(payload) {
  var self = this;

  var configResponseObj = self.ConfigResponse.parse(payload);
  self.agent.logger.log(util.inspect(configResponseObj, {depth: 20}));

  self.agent.configManager.updateConfig(configResponseObj);

  if(configResponseObj.processCallGraphReq) {
    self.protobufModel.createProcessSnapshot(configResponseObj.processCallGraphReq, function(err, processSnapshot) {
      if(err) {
        self.agent.logger.error(err);
      }
      else {
        self.sendProcessSnapshot(processSnapshot);
      }
    });
  }
}



ProxyTransport.prototype.sendBTInfoRequest = function(transaction) {
  var self = this;

  var btInfoRequest = {
    requestID: transaction.id.toString(),
    messageID: 0,
    btIdentifier: self.protobufModel.createBTIdentifier(transaction),
    correlation: self.protobufModel.createCorrelation(transaction)
  };

  // assign transaction to message id in order to find it when info response is received
  transaction.btInfoRequest = btInfoRequest;
  self.transactionMap[btInfoRequest.requestID] = transaction;


  // send to proxy
  var asyncRequestObj = {
    type: 'BTINFO',
    btInfoReq: btInfoRequest
  };


  try {
    var asyncRequestBytes = self.ASyncRequest.serialize(asyncRequestObj);

    self.infoSocket.send([
      "AsyncReqRouter",
      "",
      asyncRequestBytes
    ]);
  }
  catch(err) {
    self.agent.logger.error(err);
  }
}


ProxyTransport.prototype.receiveBTInfoResponse = function(payload) {
  var self = this;

  var btInfoResponse = self.BTInfoResponse.parse(payload);
  self.agent.logger.log("btInfoResponse");
  self.agent.logger.log(btInfoResponse);

  if(btInfoResponse.processCallGraphReq) {
    self.protobufModel.createProcessSnapshot(btInfoResponse.processCallGraphReq, function(err, processSnapshot) {
      if(err) {
        self.agent.logger.error(err);
      }
      else {
        self.sendProcessSnapshot(processSnapshot);
      }
    });
  }

  var transaction = self.transactionMap[btInfoResponse.requestID];
  if(transaction) {
    delete self.transactionMap[btInfoResponse.requestID];

    if(transaction.isSent) {
      return;
    }

    transaction.btInfoResponse = btInfoResponse;
    transaction.emit('btInfoResponse');

    // if transaction has finished
    if(transaction.ms !== undefined) {
      self.sendTransactionDetails(transaction);
    }
  }
}



ProxyTransport.prototype.sendTransactionDetails = function(transaction) {
  var self = this;

  // in case no BTInfoResponse received yet, wait for 10ms
  if(!transaction.btInfoResponse && !transaction.hasWaitedForBTInfoResponse) {
    self.agent.logger.log('waiting for btInfoResponse');

    self.agent.timers.setTimeout(function() {
      transaction.hasWaitedForBTInfoResponse = true;
      self.sendTransactionDetails(transaction);
    }, 10);

    return;
  }

  if(transaction.isSent) {
    self.agent.logger.log('Transaction has already been sent. Name: ' + transaction.name);
    return;
  }
  transaction.isSent = true;


  var messageObj = {
    type: 'BTDETAILS',
    btDetails: self.protobufModel.createBTDetails(transaction)
  };

  try {
    var asyncMessageBytes = self.ASyncMessage.serialize(messageObj);
    self.agent.logger.log('BTDetails payload');
    self.agent.logger.log(util.inspect(self.ASyncMessage.parse(asyncMessageBytes), {depth: 20}))

    self.reportingSocket.send(asyncMessageBytes);
  }
  catch(err) {
    self.agent.logger.error(err);
  }
}



ProxyTransport.prototype.sendSelfReResolution = function(selfReResolution) {
  var self = this;

  var messageObj = {
    type: 'SELFRERESOLUTION',
    selfReResolution: selfReResolution
  };

  try {
    var asyncMessageBytes = self.ASyncMessage.serialize(messageObj);
    self.agent.logger.log('SelfReResolution payload');
    self.agent.logger.log(util.inspect(self.ASyncMessage.parse(asyncMessageBytes), {depth: 20}))

    self.reportingSocket.send(asyncMessageBytes);
  }
  catch(err) {
    self.agent.logger.error(err);
  }
}


ProxyTransport.prototype.sendProcessSnapshot = function(processSnapshot) {
  var self = this;

  var messageObj = {
    type: 'PROCESSSNAPSHOT',
    processSnapshot: processSnapshot
  };

  try {
    var asyncMessageBytes = self.ASyncMessage.serialize(messageObj);
    self.agent.logger.log('ProcessSnapshot payload');
    self.agent.logger.log(util.inspect(self.ASyncMessage.parse(asyncMessageBytes), {depth: 20}))

    self.reportingSocket.send(asyncMessageBytes);
  }
  catch(err) {
    self.agent.logger.error(err);
  }
}

==> appdynamics-0.1.2.tgz: package/lib/proxy/proxy-transport.js <==
'use strict';

var util = require('util');
var os = require('os');
var fs = require('fs');
var cluster = require('cluster');
var path = require('path');
var zmq = require('appdynamics-zmq');
var Schema = require('appdynamics-protobuf').Schema;
var ProtobufModel = require('./protobuf-model').ProtobufModel;


function ProxyTransport(agent) {
  this.agent = agent;

  this.schema = undefined;
  this.controlSock = undefined;
  this.configSock = undefined;
  this.reportingSocket = undefined;
  this.infoSocket = undefined;

  this.protobufModel = undefined;
}
exports.ProxyTransport = ProxyTransport;


ProxyTransport.prototype.init = function() {
  var self = this;

  self.protobufModel = new ProtobufModel(self.agent);
  self.protobufModel.init();
  self.schema = new Schema(fs.readFileSync(path.join(__dirname, '../..', 'conf/protobuf.desc')));

  self.nextMessageId = 0;
  self.transactionMap = {};

  self.agent.on('proxyStarted', function(nodeIndex) {
    try {
      self.control(nodeIndex);
    }
    catch(err) {
      self.agent.logger.error(err);
    }
  });
}


ProxyTransport.prototype.cleanupTransactionMap = function() {
  var self = this;

  var minTs = Date.now() - 10000;
  for(var requestId in self.transactionMap) {
    var transaction = self.transactionMap[requestId];
    if(transaction.ts < minTs) {
      delete self.transactionMap[requestId];
    }
  }
}



ProxyTransport.prototype.control = function(nodeIndex) {
  var self = this;

  if(cluster.isMaster && !self.agent.opts.monitorClusterMaster) {
    // trying to identify cluster master
    var isClusterMaster = false;
    if(cluster.workers) {
      for(var id in cluster.workers) {
        isClusterMaster = true;
        break;
      }
    }

    if(isClusterMaster) {
      // do not monitor cluster master node
      return;
    }
  }


  self.agent.logger.log('starting control socket..');

  self.StartNodeRequest = self.schema['appdynamics.pb.StartNodeRequest'];
  self.StartNodeResponse = self.schema['appdynamics.pb.StartNodeResponse'];

  self.controlSock = zmq.socket('req');
  self.controlSock.connect('ipc://' + self.agent.tmpDir + '/proxy/control/0');
  self.controlSock.on('message', function(envelope, message) {
    self.agent.logger.log('control response message received from proxy');

    if(message) {
      self.receiveStartNodeResponse(message);
    }
  });
  self.controlSock.on('error', function(err) {
    self.agent.logger.error(err);
  });


  self.agent.timers.setTimeout(function() {
    self.sendStartNodeRequest(nodeIndex);
  }, 2000);
}




ProxyTransport.prototype.start = function(dataSocketDirPath, skipInitialConfigRequest) {
  var self = this;

  // proxy communication directory for zmq
  var opts = self.agent.opts;

  // init transport and response listeners
  self.ASyncRequest = self.schema['appdynamics.pb.ASyncRequest'];
  self.ConfigResponse = self.schema['appdynamics.pb.ConfigResponse'];
  self.BTInfoRequest = self.schema['appdynamics.pb.BTInfoRequest'];
  self.BTInfoResponse = self.schema['appdynamics.pb.BTInfoResponse'];
  self.ASyncMessage = self.schema['appdynamics.pb.ASyncMessage'];

  self.configSocket = zmq.socket('router');
  self.configSocket.connect('ipc://' + dataSocketDirPath + '/0');
  self.configSocket.on('message', function(envelope, empty, part1) {
    self.agent.logger.log('config response message received from proxy');
    self.receiveConfigResponse(part1);
  });
  self.configSocket.on('error', function(err) {
    self.agent.logger.error(err);
  });

  self.infoSocket = zmq.socket('router');
  self.infoSocket.connect('ipc://' + dataSocketDirPath + '/0');
  self.infoSocket.on('message', function(envelope, empty, part1) {
    self.agent.logger.log('info response message received from proxy');
    self.receiveBTInfoResponse(part1);
  });
  self.infoSocket.on('error', function(err) {
    self.agent.logger.error(err);
  });

  self.reportingSocket = zmq.socket('pub');
  self.reportingSocket.connect('ipc://' + dataSocketDirPath + '/1');
  self.reportingSocket.on('error', function(err) {
    self.agent.logger.error(err);
  });

  // send on connection, do not wait
  if(!skipInitialConfigRequest) {
    self.agent.timers.setTimeout(function() {
      self.sendConfigRequest();
    }, 2000);
  }

  // should we also reqest config after registering unregistered?
  self.agent.timers.setInterval(function() {
    self.sendConfigRequest();
  }, 30000);

  // send on connection, do not wait
  self.agent.timers.setInterval(function() {
    self.cleanupTransactionMap();
  }, 10000);
}



ProxyTransport.prototype.sendStartNodeRequest = function(nodeIndex) {
  var self = this;
  var opts = self.agent.opts;

  var computedNodeName = (opts.nodeName || os.hostname());
  if (!opts.noNodeNameSuffix)
    computedNodeName += '-' + nodeIndex;

  var startNodeRequestObj = {
    appName: opts.applicationName,
    tierName: opts.tierName,
    nodeName: computedNodeName,
    controllerHost: opts.controllerHostName,
    controllerPort: opts.controllerPort,
    sslEnabled: opts.controllerSslEnabled || false,
    accountName: opts.accountName,
    accountAccessKey: opts.accountAccessKey,
    httpProxyHost: opts.proxyHost,
    httpProxyPort: opts.proxyPort
  };

  try {
    self.agent.logger.log(util.inspect(startNodeRequestObj, {depth: 20}));
    var startNodeRequestBytes = self.StartNodeRequest.serialize(startNodeRequestObj);
    console.log(startNodeRequestBytes);
    console.log(startNodeRequestBytes.length);

    self.controlSock.send([
      "",
      startNodeRequestBytes
    ]);
  }
  catch(err) {
    self.agent.logger.error(err);
  }
}



ProxyTransport.prototype.receiveStartNodeResponse = function(payload) {
  var self = this;

  var startNodeResponseObj = self.StartNodeResponse.parse(payload);

  if(startNodeResponseObj) {
    if(startNodeResponseObj.dataSocketDirPath) {
      self.start(startNodeResponseObj.dataSocketDirPath, !!startNodeResponseObj.configResponse);
    }

    if(startNodeResponseObj.configResponse) {
      var configResponseObj = startNodeResponseObj.configResponse;

      self.agent.configManager.updateConfig(configResponseObj);
      if(configResponseObj.processCallGraphReq) {
        self.protobufModel.createProcessSnapshot(configResponseObj.processCallGraphReq, function(err, processSnapshot) {
          if(err) {
            self.agent.logger.error(err);
          }
          else {
            self.sendProcessSnapshot(processSnapshot);
          }
        });
      }
    }
  }
}


ProxyTransport.prototype.sendConfigRequest = function() {
  var self = this;

  var lastConfigVersion = self.agent.configManager.getConfigVersion();
  var processMetrics = self.agent.metricsManager.getProcessMetrics();

  var asyncRequestObj = {
    type: 'CONFIG',
    configReq: {
      lastVersion: lastConfigVersion,
      nodejsProcessMetrics: processMetrics
    }
  };

  try {
    self.agent.logger.log(util.inspect(asyncRequestObj, {depth: 20}));
    var asyncRequestBytes = self.ASyncRequest.serialize(asyncRequestObj);

    self.configSocket.send([
      "AsyncReqRouter",
      "",
      asyncRequestBytes
    ]);
  }
  catch(err) {
    self.agent.logger.error(err);
  }
}


ProxyTransport.prototype.receiveConfigResponse = function(payload) {
  var self = this;

  var configResponseObj = self.ConfigResponse.parse(payload);
  self.agent.logger.log(util.inspect(configResponseObj, {depth: 20}));

  self.agent.configManager.updateConfig(configResponseObj);

  if(configResponseObj.processCallGraphReq) {
    self.agent.processScanner.startManualSnapshot(configResponseObj.processCallGraphReq);
  }
}



ProxyTransport.prototype.sendBTInfoRequest = function(transaction) {
  var self = this;

  var btInfoRequest = {
    requestID: transaction.id.toString(),
    messageID: 0,
    btIdentifier: self.protobufModel.createBTIdentifier(transaction),
    correlation: self.protobufModel.createCorrelation(transaction)
  };

  // assign transaction to message id in order to find it when info response is received
  transaction.btInfoRequest = btInfoRequest;
  self.transactionMap[btInfoRequest.requestID] = transaction;


  // send to proxy
  var asyncRequestObj = {
    type: 'BTINFO',
    btInfoReq: btInfoRequest
  };


  try {
    var asyncRequestBytes = self.ASyncRequest.serialize(asyncRequestObj);

    self.infoSocket.send([
      "AsyncReqRouter",
      "",
      asyncRequestBytes
    ]);
  }
  catch(err) {
    self.agent.logger.error(err);
  }
}


ProxyTransport.prototype.receiveBTInfoResponse = function(payload) {
  var self = this;

  var btInfoResponse = self.BTInfoResponse.parse(payload);
  self.agent.logger.log("btInfoResponse");
  self.agent.logger.log(btInfoResponse);

  if(btInfoResponse.processCallGraphReq) {
    self.agent.processScanner.startManualSnapshot(btInfoResponse.processCallGraphReq);
  }

  var transaction = self.transactionMap[btInfoResponse.requestID];
  if(transaction) {
    delete self.transactionMap[btInfoResponse.requestID];

    if(transaction.isSent) {
      return;
    }

    transaction.btInfoResponse = btInfoResponse;
    transaction.emit('btInfoResponse');

    // if transaction has finished
    if(transaction.ms !== undefined) {
      self.sendTransactionDetails(transaction);
      return;
    }

    // If snapshot is required, try to start a process snapshot.
    if (btInfoResponse.isSnapshotRequired) {
      self.agent.processScanner.startAutoSnapshotIfPossible();
    }
  }
}



ProxyTransport.prototype.sendTransactionDetails = function(transaction) {
  var self = this;

  // in case no BTInfoResponse received yet, wait for 10ms
  if(!transaction.btInfoResponse && !transaction.hasWaitedForBTInfoResponse) {
    self.agent.logger.log('waiting for btInfoResponse');

    self.agent.timers.setTimeout(function() {
      transaction.hasWaitedForBTInfoResponse = true;
      self.sendTransactionDetails(transaction);
    }, 10);

    return;
  }

  if(transaction.isSent) {
    self.agent.logger.log('Transaction has already been sent. Name: ' + transaction.name);
    return;
  }
  transaction.isSent = true;


  var messageObj = {
    type: 'BTDETAILS',
    btDetails: self.protobufModel.createBTDetails(transaction)
  };

  try {
    var asyncMessageBytes = self.ASyncMessage.serialize(messageObj);
    self.agent.logger.log('BTDetails payload');
    self.agent.logger.log(util.inspect(self.ASyncMessage.parse(asyncMessageBytes), {depth: 20}))

    self.reportingSocket.send(asyncMessageBytes);
  }
  catch(err) {
    self.agent.logger.error(err);
  }
}



ProxyTransport.prototype.sendSelfReResolution = function(selfReResolution) {
  var self = this;

  var messageObj = {
    type: 'SELFRERESOLUTION',
    selfReResolution: selfReResolution
  };

  try {
    var asyncMessageBytes = self.ASyncMessage.serialize(messageObj);
    self.agent.logger.log('SelfReResolution payload');
    self.agent.logger.log(util.inspect(self.ASyncMessage.parse(asyncMessageBytes), {depth: 20}))

    self.reportingSocket.send(asyncMessageBytes);
  }
  catch(err) {
    self.agent.logger.error(err);
  }
}


ProxyTransport.prototype.sendProcessSnapshot = function(processSnapshot) {
  var self = this;

  var messageObj = {
    type: 'PROCESSSNAPSHOT',
    processSnapshot: processSnapshot
  };

  try {
    var asyncMessageBytes = self.ASyncMessage.serialize(messageObj);
    self.agent.logger.log('ProcessSnapshot payload');
    self.agent.logger.log(util.inspect(self.ASyncMessage.parse(asyncMessageBytes), {depth: 20}))

    self.reportingSocket.send(asyncMessageBytes);
  }
  catch(err) {
    self.agent.logger.error(err);
  }
}
